<?php

/**
 * Implementation of node_form_alter - invoked in hook_form_alter
 */

function dependent_fields_node_form_alter(&$form, &$form_state, $form_id) {
  // Get any controller fields.
  $handler = new dependent_fields_handler;
  $handler->init($form);
  dependent_fields_set_handler($form['#build_id'], $handler);

  // Only add the callbacks and overrides if we have at least one controller field.
  if (!empty($handler->tree)) {
    $processor = array('dependent_processor' => array(
      '#type' => 'dependent_processor',
      '#handler' => $handler,
    ));
    $form = $processor + $form;

    $handler->set_field_overrides($form);
    $handler->reorganize_form($form);

    $form['#after_build'][] = 'dependent_fields_node_form_after_build';
    $form['#pre_render'][] = 'dependent_fields_node_form_pre_render';
    $form['#validate'][] = 'dependent_fields_node_form_validate';

  }
}

/**
 * Additional #after_build handler for the node form.
 *
 * This function sets the caching status to true, so we can retrieve it in our
 * AHAH callback. It also moves any dependent fields into the controller field's
 * AHAH wrapper.
 */

function dependent_fields_node_form_after_build($form, &$form_state) {
  // Add CSS file to set the form styles.
  dependent_fields_add();
  $form['#cache'] = TRUE;
  $handler = $form['dependent_processor']['#handler'];
  
  // Check to see if this type has any controller fields.
  if (!empty($handler->controllers)) {
    // Check to see if this is an ahah submission.
    if (!empty($form_state['ahah_submission']) && $form_state['ahah_submission']) {
      // Disable validation on the entire form, to prevent errors on fields.
      dependent_fields_disable_validation($form);
    }
  }

  return $form;
}

/**
 * Additional #validate handler for the node form.
 */
function dependent_fields_node_form_validate($form, &$form_state) {
  $continue_button = $form_state['clicked_button']['#submit'][0] == 'dependent_fields_fallback_submit' ? TRUE : FALSE;
    if ($continue_button) {
      /// Since we can't seem to disable all validation, lets clear error messages
      drupal_get_messages();
    }
}

/**
 * Additional #pre_render handler for the node form.
 */

function dependent_fields_node_form_pre_render($form) {
  // Add CSS file to set the form styles.
  dependent_fields_add();
  return $form;
}

/**
 * Run the after build functions on the field element.
 */
function dependent_fields_callback_after_build(&$form, &$form_state) {
  if (!is_array($form)){
    return;
  }

  if (isset($form['#after_build'])) {
    foreach ($form['#after_build'] as $function) {
      $form = $function($form, $form_state);
      $form['#after_build']['#after_build_done'] = TRUE;
    }
  }
  foreach ($form as $id => $element) {
    if ($id != '#after_build' && is_array($element)) {
      dependent_fields_callback_after_build($form[$id], $form_state);
    }
  }
}

/**
 * Disable validation for all of the form.
 */
function dependent_fields_disable_validation(&$form) {
  foreach (element_children($form) as $child) {
    $form[$child]['#validated'] = TRUE;
    $form[$child]['#element_validate'] = NULL;
    $form[$child]['#needs_validation'] = FALSE;
    dependent_fields_disable_validation($form[$child]);
  }
}

/**
 * Function to check if a field is located within a fieldgroup.
 */
function dependent_fields_get_field_group_name($type, $field_name) {
  if (module_exists('fieldgroup') && ($group_name = _fieldgroup_field_get_group($type['type'], $field_name))) {
    return $group_name;
  }

  return FALSE;
}

/**
 * Function to retrieve a field's parent element from a form.
 */
function &dependent_fields_retrieve_field_parent_element(&$form, $type, $field_name) {
  $group_name = &dependent_fields_get_field_group_name($type, $field_name);

  if (!empty($group_name) && !empty($form[$group_name])) {
    $parent = &$form[$group_name];
  }
  else {
    $parent = &$form;
  }

  return $parent;
}

/**
 * Function to retrieve a field element from a form.
 */
function &dependent_fields_retrieve_field_element(&$form, $type, $field_name) {
  $parent = &dependent_fields_retrieve_field_parent_element($form, $type, $field_name);
  return $parent[$field_name];
}


/**
 * Function to retrieve a field element form from a form.
 */
function &dependent_fields_retrieve_field_form_parent(&$form, $field, $type) {
  $field_name = $field['field_name'];
 
  $group_name = dependent_fields_get_field_group_name($type, $field_name);
  $group_name = '';
  $controller_field = dependent_fields_controller_field($field, $type);
  $controlled_fields = dependent_fields_controlled_fields($field, $type);

  // Check to find out which kind of field this is.
  if (empty($controller_field) && empty($controlled_fields)) {
    // This field is not managed by dependent fields so we can simply return a regular field.
    $parent = &dependent_fields_retrieve_field_parent_element($form, $field_name, $type);
    return $parent;
  }
  elseif (empty($controller_field)) {
    if (!empty($group_name)) {
      $parent = &$form[$group_name][$field_name];
    }
    else {
      $parent = &$form[$field_name];
    }
    return $parent;
  }
  else {
    // We have controller field, so the field could be in one of several places.
    $controller_field_name = $controller_field['field_name'];

    if (!empty($group_name)) {
      $controller_field_name = $controller_field['field_name'];

      if (!empty($form[$group_name][$controller_field_name . '_ahah_wrapper'])) {
        $parent = $form[$group_name][$controller_field_name . '_ahah_wrapper'];
      }
      else {
        $parent = &dependent_fields_retrieve_field_form_parent($form, $controller_field, $type);
      }
    }
    else {
      // We have controller field, so the field could be in one of several places.
      $parent_controlled_fields = dependent_fields_controlled_fields_list($controller_field, $type);
      $parent_controller_field = dependent_fields_controller_field($controller_field, $type);
      // if there is only one controlled field, then the field is not moved.
      if (count($parent_controlled_fields) == 1 && empty($parent_controller_field)) {
        $parent = &$form[$controller_field_name . '_ahah_wrapper'];
      }
      else {
        $parent = &dependent_fields_retrieve_field_form_parent($form, $controller_field, $type);
      }
    }

    if ($parent['#processed'] && !empty($parent[$field_name])) {
      $parent = &$parent[$field_name];
    }
    else {
      $parent = &$parent['#fields'];
    }
    
    if (!empty($controlled_fields)) {
      // Multi level dependency, lets find the correct field element.
      $parent = &$parent[$field_name];
    }
    return $parent;
  }

}

/**
 * Function to retrieve a field element form from a form.
 */
function dependent_fields_retrieve_field_form(&$form, $field, $type) {
  $field_name = $field['field_name'];
  $parent = dependent_fields_retrieve_field_form_parent($form, $field, $type);
  $field_element = $parent[$field_name];
  return $field_element;
}