<?php
/**
 * @file
 *
 * Provides the basic object definitions used by dependent_fields.
 */

/**
 * Base class for dependent fields handler object.
 */
class dependent_fields_handler {
  // Private Variables
  var $allowed_widgets;

  // Dependency Tree
  var $tree;
  
  //
  // Form variables
  var $build_id = NULL;
  var $type = NULL;
  var $type_name = NULL;

  var $fields = array();
  var $field_info = array();
  var $field_values = array();
  var $field_keys = array();
  var $field_titles = array();
  var $field_overrides = array();

  /**
   * Initialize this object.
   *
   * @param $init
   *   If an array, this is FAPI Form array. Otherwsie, if a string, it must be
   *   accompanied by the type parameter.
   * @param $type
   *   If an array, this is the CCK type array. Otherwise, if a string valu, the
   *   type array will be retrieved.
   *
   */
  function init($form, $type = '') {
    // Check if we have a form array passed in.
    if (is_array($form)) {
      $this->build_id = $form['#build_id'];
      $type = empty($type) ? $form['type']['#value'] : $type;
      $this->fields = $form['#field_info'];
    }
    elseif(is_string($form) && !empty($type)) {
      $this->build_id = $form;
      $fields = content_fields(NULL, $type);
    }

    // return if there are no dependent fields to work with
    if (empty($this->fields)) return;
    
    if (is_string($type)) {
      $this->type = (array) content_types($type);
    }

    $this->type_name = $this->type['type'];
    $this->build_dependency_tree();
    $this->set_field_info();

    if (empty($init)) {
      return;
    }

  }

  /**
   * Build the dependency tree.
   */
  function build_dependency_tree($tree = NULL) {
    // Get list of allowed widgets for both field types
    $this->allowed_widgets = dependent_fields_allowed_widgets();
    $controlled = array();
    $controllers = array();
    foreach ($this->fields as $field) {
      $widget = $field['widget'];
      $widget_type = isset($widget['widget_type']) ? $widget['widget_type'] : $widget['type'];
      // Exclude fields other than those with allowed widgets
      if (!in_array($widget_type, $this->allowed_widgets)) {
        continue;
      }
      if (!empty($field['widget']['dependent_fields']['controller_field'])) {

        $controlled[$field['widget']['dependent_fields']['controller_field']][$field['field_name']] = array();
      }
      else {
        $controllers[$field['field_name']] = array();
      }
    }
    $this->_build_dependency_tree($controllers, $controlled);
    $this->tree = $controllers;
  }

  /**
   * Provide a list of controller fields enabled for a content type.
   */
  function _build_dependency_tree(&$tree, $controlled) {
    foreach ($tree as $field_name => $field_array) {
      if (!empty($controlled[$field_name]) && is_array($controlled[$field_name])) {
        $tree[$field_name] = $controlled[$field_name];
        $this->_build_dependency_tree($tree[$field_name], $controlled);
      }
    }
  }


  /**
   * Set the field info
   */
  function set_field_info($tree = NULL) {
    if (!is_array($tree)) {
      $this->field_info = array();
      $tree = $this->tree;
    }

    $form_info = &$this->field_info;

    $another_level = FALSE;
    foreach ($tree as $field_name => $field_array) {
      // Check to see if this field has already been set.
      if (empty($form_info[$field_name])) {
        $field = $this->fields[$field_name];
        $controller_field = dependent_fields_controller_field($field, $this->type);
        $controlled_fields = dependent_fields_controlled_fields($field, $this->type);
        $hide_if_empty = $field['widget']['dependent_fields']['hide_if_empty'];
        $hide_none = $field['widget']['dependent_fields']['hide_none'];

        $field_info = array(
          'field_name' => $field_name,
          'field' => $field,
          'widget' => $field['widget'],
          'hide_if_empty' => $hide_if_empty,
          'hide_none' => $hide_none,
        );

        if (!empty($controller_field)) {
          $field_info['controller_field'] = $controller_field['field_name'];
        }

        if (!empty($controlled_fields)) {
          $field_info['controlled_fields'] = array_keys($controlled_fields);
        }

        $params = array($field_info, $field, $field['widget'], $this->type);
        $field_info = array_merge($field_info, dependent_fields_invoke_field($field['type'], 'build_field_info', $params));
        $form_info[$field_name] = $field_info;
      }

      // Process any controlled fields
      if (!empty($field_array)) {
        $this->set_field_info($field_array);
      }
    }
  }

  function set_values($values = array()) {
    $this->set_field_values($values);
    $this->set_field_keys();
    $this->set_field_titles();
  }

  function set_field_values($values = array()) {
    if (!isset($this->field_values)) {
      $this->field_values = array();
    }

    $form_data = &$this->field_values;

    foreach ($this->field_info as $field_name => $field_info) {
      if (isset($values[$field_name])) {
        $form_data[$field_name] = dependent_fields_retrieve_field_value($field_info['field'], $values);
      }
    }
  }

  function set_field_keys() {
    if (!isset($this->field_keys)) {
      $this->field_keys = array();
    }

    $form_data = &$this->field_keys;

    foreach ($this->field_info as $field_name => $field_info) {
      if (isset($this->field_values[$field_name])) {
        $form_data[$field_name] = dependent_fields_retrieve_field_keys($field_info['field'], $this->field_values[$field_name]);
      }
    }
  }

  function set_field_titles() {
    if (!isset($this->field_titles)) {
      $this->field_titles = array();
    }

    $form_data = &$this->field_titles;

    foreach ($this->field_info as $field_name => $field_info) {
      if ($this->field_values[$field_name]) {
        $form_data[$field_name] = dependent_fields_retrieve_field_titles($field_info['field'], $this->field_values[$field_name]);
      }
    }
  }


  function get_field_data($field_name, $level = 0) {
    $data = array();
    $field_info = $this->field_info[$field_name];
    $data['info'] = $field_info;
    $data['field'] = $this->fields[$field_name];
    $data['values'] = $this->field_values[$field_name];
    $data['keys'] = $this->field_keys[$field_name];
    $data['titles'] = $this->field_titles[$field_name];
    
    if ($field_info['controller_field']) {
      $data['controller'] = $this->get_field_data($field_info['controller_field'], $level + 1);
    }
    if ($field_info['controlled_fields'] && empty($level)) {
      $data['controlled'] = array();
      foreach ($field_info['controlled_fields'] as $controlled_field) {
        $data['controller'][$controlled_field] = $this->get_field_data($controlled_field, $level + 1);
      }
    }
    return $data;
  }
  
  /**
   * Function to override fields on the fly, so we can use our own validation and
   * allowed option.
   */
  function set_field_overrides(&$form, $reset = FALSE) {
    $fields = &$form['#field_info'];

    // Check for reset param, and if there revert fields back to their original modules
    if ($reset && !empty($this->field_overrides)) {
      foreach ($this->field_overrides as $field => $original) {
        $fields[$field]['module'] = $original;
      }
      unset($overrides);
      return;
    }

    $this->_set_field_overrides($fields, $this->tree);
    $this->fields = $fields;
  }


  /**
   * Recursive function to override fields on the fly.
   */
  function _set_field_overrides(&$fields, $tree) {
    $another_level = FALSE;
    foreach ($tree as $controller_name => $field_array) {
      if (empty($field_array)) {
       continue;
      }
      foreach ($field_array as $field_name => $controlled_field) {
        // Check to see if this field has already been set.
        if (empty($this->field_overrides[$field_name]) && $fields[$field_name]['module'] != 'dependent_fields') {
          $this->field_overrides[$field_name] = $fields[$field_name]['module'];
          $fields[$field_name]['original_module'] = $fields[$field_name]['module'];
          $fields[$field_name]['module'] = 'dependent_fields';
          $fields[$field_name]['form_build_id'] = $this->build_id;
        }
      }
      // Process any controlled fields
      if (!empty($controlled_field)) {
        $another_level = TRUE;
      }
    }
    if ($another_level) {
      $this->_set_field_overrides($fields, $field_array);
    }
  }



  /**
   * Function to prepare form to work with the ahah callback
   */
  function reorganize_form(&$form, $tree = NULL, $level = 0) {
    if (empty($tree)) {
      $tree = $this->tree;
    }
    $fields = $this->fields;
    $type = $this->type;

    foreach ($tree as $field_name => $field_array) {
      $controller_field = $field_name;
      $child_count = count($field_array);
      // Check if there are any controlled fields.
      if ($child_count ==  0) {
        continue;
      }
      // Recursive function to reorder controlled fields that are also controllers.
      $this->reorganize_form($form, $field_array, $level + 1);

      $group_name = &dependent_fields_get_field_group_name($type, $field_name);

      if (!empty($group_name)) {
        $parent = &$form[$group_name];
      }
      else {
        $parent = &$form;
      }

      if (empty($parent) || empty($parent[$field_name])) {
        continue;
      }

      // Override add more ahah path.
      $this->override_add_more_path($parent, $field_name, $type['type']);

      $controller = array();
      $controller['#type'] = 'dependent_controller';
      $controller['#controller_field_name'] = $field_name;
      $controller['#type_name'] = $type['type'];
      $controller[$field_name] = $parent[$field_name];
      $controller['#weight'] = $controller[$field_name]['#weight'];

      // if there is only one controlled field, we don't need to move the field.
      if ($child_count == 1 && $level == 0) {
        reset($field_array);
        $controlled_field = key($field_array);
        $controlled_parent = &dependent_fields_retrieve_field_parent_element($form, $type, $controlled_field);
        $controller['#controlled_field'] = $controlled_field;
        $wrapper = array();
        $wrapper['#type'] = 'dependent_wrapper';
        $wrapper['#parent_field'] = $field_name;
        $wrapper['#field_count'] = $child_count;
        $wrapper['#fields'] = array();
        $wrapper['#fields'][$controlled_field] = $controlled_parent[$controlled_field];
        $controlled_parent[$field_name . '_ahah_wrapper'] = $wrapper;
        unset($controlled_parent[$controlled_field]);
      }
      // If there are multiple fields, or we are in a cascading dependency, move fields to parent.
      else {
        $controller['#fields'] = array();
        $group_array = array();
        $field_count = 0;
        $controller_group = dependent_fields_get_field_group_name($type, $field_name);
        foreach ($field_array as $controlled_field => $child_fields) {
          // Check if the field is part of a group. - should we check parents?
          $group_name = dependent_fields_get_field_group_name($type, $controlled_field);
          if (!empty($group_name) && $group_name != $controller_group) {
            // Add group_name to the array for later processing.
             $group_array[$group_name] = $group_name;
          }
          else {
            ++$field_count;
            // Same group as controller.
            if (!empty($group_name)) {
              $controlled_parent = &$form[$group_name];
            }
            //Just a standard field with no group.
            else {
              $controlled_parent = &$form;
            }
            // Override ahah add_more path
            $this->override_add_more_path($controlled_parent, $controlled_field, $form['type']['#value']);
            $controller['#field_count'] = $field_count;
            $controller['#fields'][$controlled_field] = $controlled_parent[$controlled_field];
            unset($controlled_parent[$controlled_field]);
          }
        }
        $group_count = count($group_array);

        // Check if we are dealing with a single group, if so we don't need to move it, just enclose in a wrapper.
        if ($field_count == 0 && $group_count == 1) {
          reset($group_array);
          $controlled_group = key($group_array);
          $controller['#controlled_group'] = $controlled_group;
          $wrapper = array();
          $wrapper['#type'] = 'dependent_wrapper';
          $wrapper['#parent_field'] = $field_name;
          $wrapper['#field_count'] = $group_count;
          $wrapper['#group_name'] = $controlled_group;
          $wrapper['#fields'] = array();
          foreach (element_children($form[$controlled_group]) as $controlled_field => $element) {
            // Override ahah add_more paths.
            $this->override_add_more_path($form[$controlled_group], $controlled_field, $form['type']['#value']);
            $wrapper['#fields'][$element] = $form[$controlled_group][$element];
            unset($form[$controlled_group][$element]);
          }
          $form[$controlled_group][$field_name . '_ahah_wrapper'] = $wrapper;
        }
        // We have multiple items so lets move the groups too.
        elseif (!empty($group_count)) {
          foreach ($group_array as $group_name) {
            // move entire group to controller wrapper.
            foreach (element_children($form[$group_name]) as $controlled_field => $element) {
              // Override ahah add_more paths.
              $this->override_add_more_path($form[$group_name], $controlled_field, $form['type']['#value']);
            }
            $controller['#field_count'] = $group_count;
            $controller['#fields'][$group_name] = $form[$group_name];
            unset($form[$group_name]);
          }
        }
      }
      // Set the controller element to our new wrapper.
      $parent[$field_name] = $controller;
    }
  }


  /**
   * Function to override an elements add more ahah wrapper.
   */
  function override_add_more_path(&$form, $field_name) {
    // Set add more overrides if add more button exists.
    if (!empty($form[$field_name][$field_name . '_add_more'])) {
      $path = 'dependent_fields/js_add_more/' . $this->type_name . '/' . $field_name;
      $form[$field_name][$field_name . '_add_more']['#ahah']['path'] = $path;
    }
  }












}
?>
